import { Injectable, Inject } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable } from 'rxjs/Observable';
import 'rxjs/add/operator/map';
/**
 * AngualrPrestService is responsible to manage all requests
 * between the Angular aplication and the pRest API
 */
var AngularPrestService = /** @class */ (function () {
    function AngularPrestService(http, config) {
        this.http = http;
        this.config = config;
        if (this.config.baseUrl.substr(-1) !== '/') {
            this.config.baseUrl += '/';
        }
    }
    /**
     * Get the token stored inside local storage
     * @returns string
     */
    AngularPrestService.prototype._getToken = function () {
        var data = JSON.parse(localStorage.getItem(this.config.localStorageData) || '{}');
        return data[this.config.tokenPath];
    };
    /**
     * Create the request options
     * @param params PrestUrlParams
     * @returns object
     */
    AngularPrestService.prototype._getRequestOptions = function (params) {
        var options = {
            headers: new HttpHeaders().set('Content-Type', 'application/json'),
            params: params
        };
        var token = this._getToken();
        if (token) {
            options.headers.set('Authorization', "Bearer " + token);
        }
        return options;
    };
    /**
     * Create a URL to make the request
     * @param table string
     * @returns string
     */
    AngularPrestService.prototype._getUrl = function (table) {
        return "" + this.config.baseUrl + table;
    };
    /**
     * Make GET request and return only one row of data or null
     * @param table string
     * @param params PrestUrlParams
     * @returns Observable<any>
     */
    AngularPrestService.prototype.get = function (table, params) {
        return this.http.get(this._getUrl(table), this._getRequestOptions(params))
            .map(function (response) { return response[0] || {}; });
    };
    /**
     * Make GET request and returns a list of data or null
     * @param table string
     * @param params PrestUrlParams
     * @returns Observable<any>
     */
    AngularPrestService.prototype.list = function (table, params) {
        return this.http.get(this._getUrl(table), this._getRequestOptions(params));
    };
    /**
     * Make PATCH request and return the object updated
     * @param table string
     * @param data any
     * @param params PrestUrlParams
     * @returns Observable<any>
     */
    AngularPrestService.prototype.update = function (table, data, params) {
        return this.http.patch(this._getUrl(table), data, this._getRequestOptions(params));
    };
    /**
     * Make POST request and return the server response or empty
     * @param table string
     * @param data any
     * @param params PrestUrlParams
     * @returns Observable<any>
     */
    AngularPrestService.prototype.create = function (table, data, params) {
        return this.http.post(this._getUrl(table), data, this._getRequestOptions(params))
            .map(function (response) { return response || {}; });
    };
    /**
     * Make DELETE request and return the server response or empty
     * @param table string
     * @param params PrestUrlParams
     * @returns Observable<any>
     */
    AngularPrestService.prototype.delete = function (table, params) {
        return this.http.delete(this._getUrl(table), this._getRequestOptions(params))
            .map(function (response) { return response || {}; });
    };
    /**
     * Create a custom query and send to the server
     * @param type PrestRequestMethod
     * @param url string
     * @param data any
     * @returns Observable<any>
     */
    AngularPrestService.prototype.customQuery = function (type, url, data) {
        switch (type.toLowerCase()) {
            case ('get'): {
                return this.http.get(url, this._getRequestOptions())
                    .map(function (response) { return response || {}; });
            }
            case ('put'): {
                return this.http.put(url, data, this._getRequestOptions())
                    .map(function (response) { return response || {}; });
            }
            case ('patch'): {
                return this.http.patch(url, data, this._getRequestOptions())
                    .map(function (response) { return response || {}; });
            }
            case ('post'): {
                return this.http.post(url, data, this._getRequestOptions())
                    .map(function (response) { return response || {}; });
            }
            case ('delete'): {
                return this.http.get(url, this._getRequestOptions())
                    .map(function (response) { return response[0] || {}; });
            }
            default: {
                return Observable.create(function (observer) { return observer.error(new Error('Method not implemented')); });
            }
        }
    };
    AngularPrestService.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    AngularPrestService.ctorParameters = function () { return [
        { type: HttpClient, },
        { type: undefined, decorators: [{ type: Inject, args: ['config',] },] },
    ]; };
    return AngularPrestService;
}());
export { AngularPrestService };
//# sourceMappingURL=angular-prest.service.js.map